"use strict";

function _typeof(obj) { "@babel/helpers - typeof"; if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _propTypes = _interopRequireDefault(require("prop-types"));

var _react = _interopRequireWildcard(require("react"));

var _reactFauxDom = _interopRequireDefault(require("react-faux-dom"));

var _d3Cloud = _interopRequireDefault(require("d3-cloud"));

var _reactFastCompare = _interopRequireDefault(require("react-fast-compare"));

var _d3Scale = require("d3-scale");

var _d3ScaleChromatic = require("d3-scale-chromatic");

var _d3Selection = require("d3-selection");

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function _getRequireWildcardCache(nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || _typeof(obj) !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

// From: https://github.com/jasondavies/d3-cloud/blob/4fc1a943d01d270e7838c97bb8ee48ca15da20be/index.js#L355-L378
function archimedeanSpiral(size) {
  var e = size[0] / size[1];
  return function (t) {
    // eslint-disable-next-line no-return-assign, no-param-reassign
    return [e * (t *= 0.1) * Math.cos(t), t * Math.sin(t)];
  };
}

function rectangularSpiral(size) {
  var dy = 4;
  var dx = dy * size[0] / size[1];
  var x = 0;
  var y = 0;
  return function (t) {
    var sign = t < 0 ? -1 : 1; // See triangular numbers: T_n = n * (n + 1) / 2.
    // eslint-disable-next-line no-bitwise

    switch (Math.sqrt(1 + 4 * sign * t) - sign & 3) {
      case 0:
        x += dx;
        break;

      case 1:
        y += dy;
        break;

      case 2:
        x -= dx;
        break;

      default:
        y -= dy;
        break;
    }

    return [x, y];
  };
}

var spirals = {
  archimedean: archimedeanSpiral,
  rectangular: rectangularSpiral
};

function WordCloud(props) {
  var _spirals$spiral;

  var elementRef = (0, _react.useRef)();

  if (!elementRef.current) {
    elementRef.current = _reactFauxDom["default"].createElement('div');
  }

  var el = elementRef.current;
  var data = props.data,
      width = props.width,
      height = props.height,
      font = props.font,
      fontStyle = props.fontStyle,
      fontWeight = props.fontWeight,
      fontSize = props.fontSize,
      rotate = props.rotate,
      spiral = props.spiral,
      padding = props.padding,
      random = props.random,
      fill = props.fill,
      onWordClick = props.onWordClick,
      onWordMouseOver = props.onWordMouseOver,
      onWordMouseOut = props.onWordMouseOut; // clear old words

  (0, _d3Selection.select)(el).selectAll('*').remove(); // render based on new data

  var layout = (0, _d3Cloud["default"])().words(data).size([width, height]).font(font).fontStyle(fontStyle).fontWeight(fontWeight).fontSize(fontSize).rotate(rotate).spiral(typeof spiral === 'string' ? (_spirals$spiral = spirals[spiral]) !== null && _spirals$spiral !== void 0 ? _spirals$spiral : spirals.archimedean : spiral).padding(padding).random(random).on('end', function (words) {
    var texts = (0, _d3Selection.select)(el).append('svg').attr('width', layout.size()[0]).attr('height', layout.size()[1]).append('g').attr('transform', "translate(".concat(layout.size()[0] / 2, ",").concat(layout.size()[1] / 2, ")")).selectAll('text').data(words).enter().append('text').style('font-family', function (d) {
      return d.font;
    }).style('font-style', function (d) {
      return d.style;
    }).style('font-weight', function (d) {
      return d.weight;
    }).style('font-size', function (d) {
      return "".concat(d.size, "px");
    }).style('fill', fill).attr('text-anchor', 'middle').attr('transform', function (d) {
      return "translate(".concat([d.x, d.y], ")rotate(").concat(d.rotate, ")");
    }).text(function (d) {
      return d.text;
    });

    if (onWordClick) {
      texts.on('click', onWordClick);
    }

    if (onWordMouseOver) {
      texts.on('mouseover', onWordMouseOver);
    }

    if (onWordMouseOut) {
      texts.on('mouseout', onWordMouseOut);
    }
  });
  layout.start();
  return el.toReact();
}

WordCloud.propTypes = {
  data: _propTypes["default"].arrayOf(_propTypes["default"].shape({
    text: _propTypes["default"].string.isRequired,
    value: _propTypes["default"].number.isRequired
  })).isRequired,
  width: _propTypes["default"].number,
  height: _propTypes["default"].number,
  font: _propTypes["default"].oneOfType([_propTypes["default"].string, _propTypes["default"].func]),
  fontStyle: _propTypes["default"].oneOfType([_propTypes["default"].string, _propTypes["default"].func]),
  fontWeight: _propTypes["default"].oneOfType([_propTypes["default"].string, _propTypes["default"].func]),
  fontSize: _propTypes["default"].func,
  rotate: _propTypes["default"].func,
  spiral: _propTypes["default"].oneOfType([_propTypes["default"].string, _propTypes["default"].func]),
  padding: _propTypes["default"].oneOfType([_propTypes["default"].number, _propTypes["default"].func]),
  random: _propTypes["default"].func,
  fill: _propTypes["default"].func,
  onWordClick: _propTypes["default"].func,
  onWordMouseOut: _propTypes["default"].func,
  onWordMouseOver: _propTypes["default"].func
};
WordCloud.defaultProps = {
  width: 700,
  height: 600,
  font: 'serif',
  fontStyle: 'normal',
  fontWeight: 'normal',
  fontSize: function fontSize(d) {
    return Math.sqrt(d.value);
  },
  // eslint-disable-next-line no-bitwise
  rotate: function rotate() {
    return (~~(Math.random() * 6) - 3) * 30;
  },
  spiral: 'archimedean',
  padding: 1,
  random: Math.random,
  fill: function fill(d, i) {
    return (0, _d3Scale.scaleOrdinal)(_d3ScaleChromatic.schemeCategory10)(i);
  },
  onWordClick: null,
  onWordMouseOver: null,
  onWordMouseOut: null
};

var _default = /*#__PURE__*/_react["default"].memo(WordCloud, _reactFastCompare["default"]);

exports["default"] = _default;